#!/usr/bin/perl -w

# Copyright 2014, 2017, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.010;
use strict;
use FindBin;
use Math::Trig 'pi';
use Math::Libm 'hypot';
use POSIX 'fmod';
use Math::NumSeq::PlanePathTurn;
use Math::PlanePath::RationalsTree;

use FindBin;
use lib $FindBin::Bin;
use MyRationals ':all';


my $depth = 4;
my $distance_em = .5;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand\\MyDepth{$depth}\n";

my $path = Math::PlanePath::RationalsTree->new(tree_type => 'Bird');
my ($n_lo, $n_hi) = $path->tree_depth_to_n_range($depth);

my $seq = Math::NumSeq::PlanePathTurn->new(planepath_object => $path,
                                           turn_type => 'Left');

my $text = '';
my @test;
my ($x,$y) = $path->n_to_xy($n_hi);
my $prev;
foreach my $n ($n_lo .. $n_hi) {
  my ($x,$y) = $path->n_to_xy($n);
  my $is_left = $seq->ith($n);
  my $turn = ($is_left ? 'L' : 'R');

  if ($n > $n_lo && $n%4==0) { $text .= '\\\\'; }
  $text .= $turn;
  push @test, $is_left ? 1 : -1;

  my $name = xy_to_node_name($x,$y);
  print "  \\node ($name) at ($x,$y) [circle,draw,font=\\tiny] {$turn};\n";
  if ($prev) {
    print "  \\draw[->] ($prev) -- ($name);\n";
  }
  $prev = $name;
}

{
  my $n = $n_lo;
  my ($x,$y) = $path->n_to_xy($n);
  my ($prev_x,$prev_y) = $path->n_to_xy($n-1);
  my $dx = $prev_x - $x;
  my $dy = $prev_y - $y;
  my $len = hypot($dx,$dy);
  my $factor = 2.8/$len;
  $prev_x = sprintf '%.2f', $x + $dx*$factor;
  $prev_y = sprintf '%.2f', $y + $dy*$factor;
  my $name = xy_to_node_name($x,$y);
  print "  \\draw[->,dashed] ($prev_x,$prev_y) -- ($name);\n";
}
{
  my $n = $n_hi;
  my ($x,$y) = $path->n_to_xy($n);
  my ($next_x,$next_y) = $path->n_to_xy($n+1);
  my $dx = $next_x - $x;
  my $dy = $next_y - $y;
  my $len = hypot($dx,$dy);
  my $factor = 1.5/$len;
  $next_x = sprintf '%.2f', $x + $dx*$factor;
  $next_y = sprintf '%.2f', $y + $dy*$factor;
  my $name = xy_to_node_name($x,$y);
  print "  \\draw[->,dashed] ($name) -- ($next_x,$next_y);\n";
}

my $test = join(',',@test);
print <<"HERE";
  \\newcommand\\MyText{$text}
% GP-Test  vector(2^$depth,n,n+=2^$depth-1; BirdTurn_by_points(n)) == \\
% GP-Test    [$test]
  % end generated
HERE
exit 0;
