#!/usr/bin/perl -w

# Copyright 2014, 2017, 2018, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


# Print a tikz picture showing how the SB tree convex hull.

use 5.010;
use strict;
use List::Util 'max','min';
use Math::Geometry::Planar;
use Math::PlanePath::RationalsTree;

use FindBin;
use lib $FindBin::Bin;
use MyPrintwrap ':all';


my $depth = 5;

print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::RationalsTree->new(tree_type => 'SB');

my $points = [];  # arrayref of arrayrefs [$x,$y]
foreach my $n ($path->n_start .. $path->tree_depth_to_n_end($depth)) {
  my ($x,$y) = $path->n_to_xy($n);
  push @$points, [$x,$y];
}
my $planar = Math::Geometry::Planar->new;
$planar->points($points);
if (@$points > 3) { $planar = $planar->convexhull2; }
$points = $planar->points;

my @xy_is_hull;
foreach my $p (@$points) {
  my ($x,$y) = @$p;
  $xy_is_hull[$x][$y] = 1;
}

$Printwrap = 2;
printwrap("% hull boundary\n");
printwrap("\\draw ");
$Printwrap += 2;
foreach my $p (@$points) {
  my ($x,$y) = @$p;
  my $join = ($p == $points->[0] ? '' : "--");
  printwrap("$join($x,$y)");
}
printwrap("--cycle;");
printwrap_newline();

$Printwrap = 2;
print "  % hull vertices\n";
foreach my $p (@$points) {
  my ($x,$y) = @$p;
  printwrap(" \\node at ($x,$y) [my hull vertex dot];");
}
printwrap_newline();

print "  % interior points\n";
foreach my $n ($path->n_start .. $path->tree_depth_to_n_end($depth)) {
  my ($x,$y) = $path->n_to_xy($n);
  if (! $xy_is_hull[$x][$y]) {
    printwrap(" \\node at ($x,$y) [my interior dot];");
  }
}
printwrap_newline();

# descent lines
# printwrap("  \\draw");
# foreach my $n ($path->n_start .. $path->tree_depth_to_n_end($depth-1)) {
#   my ($x,$y) = $path->n_to_xy($n);
#   foreach my $child_n ($path->tree_n_children($n)) {
#     my ($child_x,$child_y) = $path->n_to_xy($child_n);
#     printwrap("  ($x,$y)--($child_x,$child_y)");
#   }
# }
# printwrap_newline();

print "  % end generated\n";
exit 0;
