#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


# Print a tikz picture showing a minimum area rectangle around the SB tree
# points, being all points rows 0 to $depth.

use 5.010;
use strict;
use List::Util 'max','min';
use Math::Geometry::Planar;
use Math::NumSeq::Fibonacci;
use Math::PlanePath::RationalsTree;

use FindBin;
use lib $FindBin::Bin;
use MyRationals;
use MyPrintwrap ':all';


my $depth = 5;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyDepth}{$depth}\n";

my $path = Math::PlanePath::RationalsTree->new(tree_type => 'SB');

$Printwrap = 2;
printwrap("\\fill foreach \\p in {");
$Printwrap += 2;
my $points;
my $x_max = 0;
my $y_max = 0;
foreach my $n ($path->n_start .. $path->tree_depth_to_n_end($depth)) {
  my ($x,$y) = $path->n_to_xy($n);
  printwrap("($x,$y),");

  push @$points, [$x,$y];
  $x_max = max($x_max, $x);
  $y_max = max($x_max, $y);
}
printwrap("}");
printwrap(" { \\p circle(\\MyCircle) };\n");

print "  \\coordinate (top-right) at ($x_max,$y_max);\n";
print "  % end generated\n";
exit 0;
